package org.jvnet.hypercompass.addon;

import java.io.File;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jvnet.hypercompass.compass.configuration.Compass;
import org.jvnet.hypercompass.compass.configuration.CompassCoreConfiguration;
import org.jvnet.hypercompass.compass.configuration.Mapping;
import org.jvnet.hypercompass.compass.configuration.impl.CompassCoreConfigurationImpl;
import org.jvnet.hypercompass.compass.configuration.impl.CompassImpl;
import org.jvnet.hypercompass.compass.configuration.impl.MappingImpl;
import org.jvnet.hypercompass.compass.mapping.CompassCoreMapping;
import org.jvnet.hypercompass.compass.mapping.strategy.DefaultPrincipalStrategy;
import org.jvnet.hypercompass.compass.mapping.strategy.PrincipalStrategy;
import org.jvnet.hypercompass.compass.mapping.utils.CompassConfigurationUtils;
import org.jvnet.hypercompass.compass.mapping.utils.CompassMappingUtils;
import org.jvnet.hypercompass.customizations.Constants;
import org.jvnet.hypercompass.runtime.tests.RoundtripTest;
import org.jvnet.jaxbcommons.addon.AbstractApplicationContextAwareCodeAugmenter;
import org.jvnet.jaxbcommons.util.CodeModelUtils;
import org.jvnet.jaxbcommons.util.GeneratorContextUtils;
import org.xml.sax.ErrorHandler;

import com.sun.msv.grammar.ExpressionVisitor;
import com.sun.tools.xjc.CodeAugmenterEx;
import com.sun.tools.xjc.Options;
import com.sun.tools.xjc.generator.ClassContext;
import com.sun.tools.xjc.generator.GeneratorContext;
import com.sun.tools.xjc.grammar.AnnotatedGrammar;
import com.sun.tools.xjc.grammar.ClassItem;

public class AddOn extends AbstractApplicationContextAwareCodeAugmenter implements CodeAugmenterEx {

  private String rt = null;

  public String getRt() {
    return rt;
  }

  public void setRt(String rt) {
    this.rt = rt;
  }

  private File target;

  public File getTarget() {
    return target;
  }

  public void setTarget(File target) {
    this.target = target;
    logger.debug("Target directory [" + target.getAbsolutePath() + "].");
  }

  private boolean cfg = true;

  public boolean isCfg() {
    return cfg;
  }

  public void setCfg(boolean cfg) {
    this.cfg = cfg;
  }

  /**
   * Logger.
   */
  protected Log logger = LogFactory.getLog(getClass());

  protected PrincipalStrategy principalStrategy = new DefaultPrincipalStrategy();

  public String getOptionName() {
    return "Xhypercompass";
  }

  public String getUsage() {
    return "  -Xhypercompass";
  }

  public List getCustomizationURIs() {
    return Collections.singletonList(Constants.NAMESPACE_URI);
  }

  public boolean run(
      AnnotatedGrammar grammar,
      GeneratorContext generatorContext,
      Options options,
      ErrorHandler errorHandler) {

    final Map map = generateMappings(grammar, generatorContext, options, errorHandler);

    marshallMappings(grammar, generatorContext, options, errorHandler, map);

    if (isCfg()) {
      final CompassCoreConfiguration configuration = generateConfiguration(
          grammar,
          generatorContext,
          options,
          errorHandler,
          map);
      marshallConfiguration(grammar, generatorContext, options, errorHandler, configuration);
    }

    if (getRt() != null) {
      GeneratorContextUtils.generateContextPathAwareClass(
          grammar,
          generatorContext,
          getRt(),
          RoundtripTest.class);
    }
    return true;
  }

  protected Map generateMappings(
      AnnotatedGrammar grammar,
      GeneratorContext generatorContext,
      Options options,
      ErrorHandler errorHandler) {
    final ClassItem[] classItems = generatorContext.getGrammar().getClasses();
    final Map map = new HashMap(classItems.length);
    for (int index = 0; index < classItems.length; index++) {
      final ClassItem classItem = classItems[index];
      final ClassContext classContext = generatorContext.getClassContext(classItem);
      final CompassCoreMapping compassCoreMapping = generateMapping(classContext);
      if (compassCoreMapping != null) {
        map.put(classItem, compassCoreMapping);
      }

      // if (compassCoreMapping != null) {
      // marshallMapping(grammar, classContext, options, errorHandler,
      // compassCoreMapping);
      // }
    }
    return map;
  }

  protected void marshallMappings(
      AnnotatedGrammar grammar,
      GeneratorContext generatorContext,
      Options options,
      ErrorHandler errorHandler,
      final Map map) {
    for (Iterator iterator = map.entrySet().iterator(); iterator.hasNext();) {
      final Entry entry = (Entry) iterator.next();
      final ClassItem classItem = (ClassItem) entry.getKey();
      final ClassContext classContext = generatorContext.getClassContext(classItem);
      final CompassCoreMapping mapping = (CompassCoreMapping) entry.getValue();
      marshallMapping(grammar, classContext, options, errorHandler, mapping);
    }
  }

  private void marshallMapping(
      AnnotatedGrammar grammar,
      final ClassContext classContext,
      Options options,
      ErrorHandler errorHandler,
      final CompassCoreMapping hibernateMapping) {
    try {
      final File target = getTarget() == null ? options.targetDir : getTarget();
      final File packageDir = new File(target, classContext.ref._package().name().replace(
          '.',
          File.separatorChar));
      if (!packageDir.exists()) {
        packageDir.mkdirs();
      }
      final File mappingFile = new File(packageDir, CodeModelUtils.getLCN(classContext.ref, false)
          + ".cpm.xml");
      CompassMappingUtils.marshall(hibernateMapping, mappingFile);
    }
    catch (Exception ex) {
      throw new RuntimeException(ex);
    }
  }

  private CompassCoreMapping generateMapping(final ClassContext classContext) {
    final ClassItem classItem = classContext.target;
    logger.debug("Processing class item [" + classItem.name + "].");

    final ExpressionVisitor visitor = (ExpressionVisitor) getPrincipalStrategy()
        .getVisitorStrategy()
        .generate(classContext);
    final CompassCoreMapping mapping = (CompassCoreMapping) classItem.visit(visitor);
    return mapping;
  }

  public PrincipalStrategy getPrincipalStrategy() {
    return principalStrategy;
  }

  public void setPrincipalStrategy(PrincipalStrategy principalStrategy) {
    this.principalStrategy = principalStrategy;
  }

  protected CompassCoreConfiguration generateConfiguration(
      AnnotatedGrammar grammar,
      GeneratorContext generatorContext,
      Options options,
      ErrorHandler errorHandler,
      Map map) {
    final CompassCoreConfiguration configuration = new CompassCoreConfigurationImpl();
    final Compass compass = new CompassImpl();
    configuration.setCompass(compass);

    //		final Setting setting = new SettingImpl();
    //		setting.setName("compass.converter.enumConverter.type");
    //		setting.setContent("org.jvnet.hypercompass.compass.core.converter.basic.EnumConverter");
    //		compass.getSetting().add(setting);

    for (Iterator iterator = map.entrySet().iterator(); iterator.hasNext();) {
      final Entry entry = (Entry) iterator.next();
      final ClassItem classItem = (ClassItem) entry.getKey();
      final ClassContext classContext = generatorContext.getClassContext(classItem);

      final String resourceName = CodeModelUtils.getFQCN(classContext.ref, false).replace('.', '/')
          + ".cpm.xml";
      final Mapping mapping = new MappingImpl();
      compass.getMapping().add(mapping);
      mapping.setResource(resourceName);
    }
    return configuration;
  }

  protected void marshallConfiguration(
      AnnotatedGrammar grammar,
      final GeneratorContext generatorContext,
      Options options,
      ErrorHandler errorHandler,
      final CompassCoreConfiguration configuration) {
    try {
      final File target = getTarget() == null ? options.targetDir : getTarget();
      final File configurationFile = new File(target, "compass.cfg.xml");
      CompassConfigurationUtils.marshall(configuration, configurationFile);
    }
    catch (Exception ex) {
      throw new RuntimeException(ex);
    }
  }

}
