package org.jvnet.hypercompass.compass.mapping.strategy;

import org.compass.core.converter.ConverterLookup;


public interface PrincipalStrategy {

  /**
   * Returns naming strategy. This strategy is responsible
   * for returning aliases, prefixes and so on.
   * @return Naming strategy.
   */
  public NamingStrategy getNamingStrategy();

  /**
   * Returns Compass core mapping strategy. Responsible for generating
   * <code>compass-core-mapping</code> elements.
   * @return Compass core mapping strategy.
   */
  public ClassStrategy getCompassCoreMappingStrategy();

  /**
   * Returns id generation strategy. Responsible for generating <code>id</code>
   * elements.
   * @return Id generation strategy.
   */
  public FieldStrategy getIdStrategy();

  /**
   * Returns property strategy. Responsible for generating
   * <code>property</code> elements.
   * @return Property strategy.
   */
  public FieldStrategy getPropertyStrategy();

  /**
   * Returns component strategy. Responsible for generating
   * <code>component</code> elements.
   * @return Component strategy.
   */

  public FieldStrategy getComponentStrategy();

  public FieldStrategy getReferenceStrategy();

  /**
   * Returns metadata generation strategy. Responsible for generating
   * <code>meta-data</code> elements.
   * @return Metadata generation strategy.
   */
  public FieldStrategy getMetaDataStrategy();

  //
  public ClassStrategy getClassStrategy();

  //
  public ClassStrategy getSubclassStrategy();

  public ClassStrategy getIdentifierStrategy();

  public ClassStrategy getFieldsStrategy();

  public FieldStrategy getPrimitiveSingleStrategy();

  public FieldStrategy getDomSingleStrategy();

  public FieldStrategy getEnumSingleStrategy();

  public FieldStrategy getWildcardSingleStrategy();

  public FieldStrategy getComplexSingleStrategy();

  public FieldStrategy getHeteroSingleStrategy();

  public FieldStrategy getComplexCollectionStrategy();

  public FieldStrategy getDomCollectionStrategy();

  public FieldStrategy getEnumCollectionStrategy();

  public FieldStrategy getPrimitiveCollectionStrategy();

  public FieldStrategy getWildcardCollectionStrategy();

  public FieldStrategy getHeteroCollectionStrategy();
  
  public ClassStrategy getVisitorStrategy();
  
//  public ConverterLookup getConverterLookup();
}
