package org.jvnet.hypercompass.compass.mapping.strategy.identifier;

import org.jvnet.hypercompass.compass.mapping.Id;
import org.jvnet.hypercompass.compass.mapping.strategy.AbstractClassStrategy;
import org.jvnet.hypercompass.compass.mapping.strategy.PrincipalStrategy;
import org.jvnet.hypercompass.customizations.Constants;
import org.jvnet.hypercompass.customizations.Utils;
import org.jvnet.hypercompass.customizations.impl.IdImpl;
import org.jvnet.jaxbcommons.util.CustomizationUtils;
import org.jvnet.jaxbcommons.util.FieldUtils;

import com.sun.codemodel.JCodeModel;
import com.sun.codemodel.JType;
import com.sun.msv.datatype.xsd.LongType;
import com.sun.msv.datatype.xsd.XSDatatype;
import com.sun.tools.xjc.generator.ClassContext;
import com.sun.tools.xjc.grammar.FieldItem;
import com.sun.tools.xjc.reader.xmlschema.bindinfo.BIXPluginCustomization;

public class DefaultIdentifierStrategy extends AbstractClassStrategy {

  public DefaultIdentifierStrategy(PrincipalStrategy principalStrategy) {
    super(principalStrategy);
  }

  public Object generate(ClassContext classContext) {

    final FieldItem draftFieldItem;
    final FieldItem draftHypercompassFieldItem = CustomizationUtils
        .getFieldItemWithCustomization(classContext, Constants.ID);
    if (draftHypercompassFieldItem != null) {
      draftFieldItem = draftHypercompassFieldItem;
    }
    else {
      final FieldItem draftHyperjaxb2FieldItem = CustomizationUtils
          .getFieldItemWithCustomization(classContext, Constants.HYPERJAXB2_ID);

      if (draftHyperjaxb2FieldItem != null) {
        draftFieldItem = draftHyperjaxb2FieldItem;
      }
      else {
        draftFieldItem = null;
      }
    }

    final FieldItem fieldItem;

    if (draftFieldItem == null) {
      final XSDatatype propertyDatatype = getPropertyDatatype();
      final String defaultIdentifierColumnName = getDefaultPropertyName();
      final JType propertyClass = getPropertyClass(classContext.parent.getCodeModel());
      fieldItem = FieldUtils.createPrimitiveFieldItem(
          classContext,
          defaultIdentifierColumnName,
          propertyDatatype,
          propertyClass);
      final org.jvnet.hypercompass.customizations.Id cid = new IdImpl();
      cid.setValue("true");
      final BIXPluginCustomization cidc = CustomizationUtils.createCustomization(Utils
          .getCustomizationElement(cid));
      fieldItem.declarations.add(cidc);
    }
    else {
      fieldItem = draftFieldItem;
    }
    

    final Id id = (Id) getPrincipalStrategy().getIdStrategy().generate(classContext, fieldItem);

    return id;
  }

  public XSDatatype getPropertyDatatype() {
    return LongType.theInstance;
  }

  public JType getPropertyClass(JCodeModel codeModel) {
    return codeModel.LONG.getWrapperClass();
  }

  public String getDefaultPropertyName() {
    return "Hcid";
  }
}
