package org.jvnet.hypercompass.compass.visitor;

import org.jvnet.hypercompass.compass.mapping.strategy.PrincipalStrategy;
import org.jvnet.hypercompass.customizations.Utils;
import org.jvnet.jaxbcommons.util.FieldUtils;
import org.jvnet.jaxbcommons.visitor.ClassifyingVisitor;

import com.sun.tools.xjc.generator.ClassContext;
import com.sun.tools.xjc.grammar.ClassItem;
import com.sun.tools.xjc.grammar.FieldItem;

public class CompassCoreMappingGeneratingVisitor extends ClassifyingVisitor {

  private final PrincipalStrategy principalStrategy;

  public PrincipalStrategy getPrincipalStrategy() {
    return principalStrategy;
  }

  public CompassCoreMappingGeneratingVisitor(
      PrincipalStrategy principalStrategy,
      ClassContext classContext) {
    super(classContext);
    this.principalStrategy = principalStrategy;
  }

  public Object onClass(ClassItem classItem) {
    if (!Utils.isIgnored(classItem)) {
      return super.onClass(classItem);
    }
    else {
      return null;
    }
  }

  public Object onClassInternal(ClassItem classItem) {

    if (classItem.getSuperClass() == null) {
      return getPrincipalStrategy().getClassStrategy().generate(classContext);
    }
    else {
      return getPrincipalStrategy().getSubclassStrategy().generate(classContext);
    }
  }

  public Object onField(FieldItem fieldItem) {
    if (Utils.isIgnored(fieldItem)) {
      return null;
    }
    else if (FieldUtils.isConstantField(classContext, fieldItem)) {
      return null;
    }
    else {
      return super.onField(fieldItem);
    }
  }

  public Object onPrimitiveSingleField(FieldItem fieldItem) {
    return getPrincipalStrategy().getPrimitiveSingleStrategy().generate(classContext, fieldItem);
  }

  public Object onDomSingleField(FieldItem fieldItem) {
    return getPrincipalStrategy().getDomSingleStrategy().generate(classContext, fieldItem);
  }

  public Object onEnumSingleField(FieldItem fieldItem) {
    return getPrincipalStrategy().getEnumSingleStrategy().generate(classContext, fieldItem);
  }

  public Object onWildcardSingleField(FieldItem fieldItem) {
    return getPrincipalStrategy().getWildcardSingleStrategy().generate(classContext, fieldItem);
  }

  public Object onHeteroSingleField(FieldItem fieldItem) {
    return getPrincipalStrategy().getHeteroSingleStrategy().generate(classContext, fieldItem);
  }

  public Object onComplexSingleField(FieldItem fieldItem) {
    return getPrincipalStrategy().getComplexSingleStrategy().generate(classContext, fieldItem);
  }

  public Object onPrimitiveCollectionField(FieldItem fieldItem) {
    return getPrincipalStrategy()
        .getPrimitiveCollectionStrategy()
        .generate(classContext, fieldItem);
  }

  public Object onDomCollectionField(FieldItem fieldItem) {
    return getPrincipalStrategy().getDomCollectionStrategy().generate(classContext, fieldItem);
  }

  public Object onEnumCollectionField(FieldItem fieldItem) {
    return getPrincipalStrategy().getEnumCollectionStrategy().generate(classContext, fieldItem);
  }

  public Object onWildcardCollectionField(FieldItem fieldItem) {
    return getPrincipalStrategy().getWildcardCollectionStrategy().generate(classContext, fieldItem);
  }

  public Object onComplexCollectionField(FieldItem fieldItem) {
    return getPrincipalStrategy().getComplexCollectionStrategy().generate(classContext, fieldItem);
  }

  public Object onHeteroCollectionField(FieldItem fieldItem) {
    return getPrincipalStrategy().getHeteroCollectionStrategy().generate(classContext, fieldItem);
  }
}
